<?php

namespace Drupal\flashpoint_commerce_course\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\commerce_product\Entity\Product;
use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\group\Entity\Group;
use Drupal\group\Entity\GroupContent;
use Drupal\group\GroupMembership;
use Drupal\flashpoint_course\FlashpointCourseUtilities;
use Drupal\state_machine\Event\WorkflowTransitionEvent;

/**
 * Class OrderCompleteSubscriber.
 *
 * @package Drupal\flashpoint_commerce
 */
class FlashpointCommerceOrderCompleteSubscriber implements EventSubscriberInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManager definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Constructor.
   */
  public function __construct(EntityTypeManager $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  static function getSubscribedEvents() {
    $events['commerce_order.place.post_transition'] = ['orderCompleteHandler'];

    return $events;
  }

  /**
   * This method is called whenever the commerce_order.place.post_transition event is
   * dispatched.
   *
   * @param WorkflowTransitionEvent $event
   */
  public function orderCompleteHandler(WorkflowTransitionEvent $event) {
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $event->getEntity();

    // Order items in the cart.
    $items = $order->getItems();

    foreach ($items as $item) {
      $product = FALSE;
      $type = $item->getPurchasedEntity()->getEntityTypeId();
      if ($type === 'commerce_product_variation') {
        $var = ProductVariation::load($item->getPurchasedEntity()->id());
        $product = $var->getProduct();
      }
      else {
        $item->getPurchasedEntity();
      }
      switch($product->bundle()) {
        case 'course':
          $item_id = $item->id();
          $course_id = $product->get('field_course')->getValue()[0]['target_id'];
          $course = Group::load($course_id);
          $membership = $course->getMember($order->getCustomer());
          $access = FlashpointCourseUtilities::enrollAccess($course, $order->getCustomer());
          if ($access) {
            if (!$membership) {
              $course->addMember($order->getCustomer(), ['field_redeemed_order_items' => [$item_id]]);
            }
            else {
              $gc = $membership->getGroupContent();
              $current_values = $gc->get('field_redeemed_order_items')->getValue();
              $set = [$item_id];
              foreach ($current_values as $posn => $val) {
                $set[] = $val['target_id'];
              }
              $gc->set('field_redeemed_order_items', $set);
              $gc->save();
            }
          }
        break;
      }
    }
  }
}